package it.unimi.di.archive4j;

/*
 * Copyright (C) 2008-2013 Alessio Orlandi and Sebastiano Vigna
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


import it.unimi.dsi.fastutil.ints.Int2IntLinkedOpenHashMap;
import it.unimi.dsi.fastutil.ints.Int2ObjectLinkedOpenHashMap;
import it.unimi.dsi.fastutil.ints.IntArrayList;

import java.io.IOException;
import java.util.Iterator;
import java.util.NoSuchElementException;



/**
 * Dynamic implementation of an in-memory archive. Supports random access.
 * 
 * <p>Just append summaries and query, in any order.
 * 
 * @author Alessio Orlandi
 */

public class ArrayArchive extends ArchiveWriter<ArrayDocumentSummary> implements Archive<ArrayDocumentSummary> {

	protected Int2ObjectLinkedOpenHashMap<ArrayDocumentSummary> data;

	protected IntArrayList ordinal;

	protected int numberOfWords;

	protected Int2IntLinkedOpenHashMap frequency;

	public ArrayArchive() {
		data = new Int2ObjectLinkedOpenHashMap<ArrayDocumentSummary>();
		ordinal = new IntArrayList();
		numberOfWords = 0;
		frequency = new Int2IntLinkedOpenHashMap();
	}

	@Override
	public void append( ArrayDocumentSummary summary ) throws IOException {
		if ( data.get( summary.id() ) != null ) throw new IllegalStateException( "Duplicate document id " + summary.id() );
		data.put( summary.id(), summary );
		ordinal.add( summary.id() );
		numberOfWords += summary.length();

		for ( int i = 0; i < summary.size(); i++ )
			frequency.put( summary.term( i ), frequency.get( summary.term( i ) ) + summary.count( i ) );
	}

	public int frequency( int term ) {
		return frequency.get( term );
	}

	public ArrayDocumentSummary getDocumentById( int id ) throws IOException {
		return data.get( id );
	}

	public ArrayDocumentSummary getDocumentByIndex( int idx ) throws IOException {
		if ( idx >= 0 && idx < ordinal.size() ) return getDocumentById( ordinal.getInt( idx ) );

		throw new NoSuchElementException( "Out of bounds: " + idx );
	}

	public int numberOfDocuments() {
		return ordinal.size();
	}

	public int numberOfTerms() {
		return frequency.size();
	}

	public long numberOfWords() {
		return numberOfWords;
	}

	public void close() throws IOException {}

	public Iterator<ArrayDocumentSummary> iterator() {
		return data.values().iterator();
	}

	public boolean hasRandomAccess() {
		return true;
	}

	public Archive<ArrayDocumentSummary> copy() {
		return this;
	}
}
