package it.unimi.di.archive4j;

/*		 
 * Copyright (C) 2008-2013 Alessio Orlandi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


import it.unimi.di.archive4j.Archive;
import it.unimi.di.archive4j.ArchiveLoader;
import it.unimi.di.archive4j.ArrayDocumentSummary;
import it.unimi.di.archive4j.RandomAccessBitstreamArchive;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags.Coding;
import it.unimi.di.archive4j.SequentialBitstreamArchive.CompressionFlags.Component;
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap;
import junit.framework.TestCase;

public class RandomAccessBitstreamArchiveTest extends TestCase {

	protected String basename;

	protected boolean notTearDown;
	protected String basenameNoCounts;
	protected String basenameNoDocLengths;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		final Object2ObjectOpenHashMap<Component, Coding> flags = new Object2ObjectOpenHashMap<Component, Coding>( CompressionFlags.DEFAULT );
		basename = TestCollection.buildRandomAccessArchive( flags );
		flags.put( Component.COUNTS, null );
		basenameNoCounts = TestCollection.buildRandomAccessArchive( flags );
		flags.putAll( CompressionFlags.DEFAULT );
		flags.put( Component.DOCLENGTHS, null );
		basenameNoDocLengths = TestCollection.buildRandomAccessArchive( flags );
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
		if ( ! notTearDown ) {
			TestCollection.cleanupBitstreamArchive( basename );
			TestCollection.cleanupBitstreamArchive( basenameNoCounts );
			TestCollection.cleanupBitstreamArchive( basenameNoDocLengths );
		}
	}

	public void testByIndex() throws Exception {
		for( boolean b: new boolean[] { true, false } ) {
			for( boolean copy: new boolean[] { false, true } ) {
				System.err.println( b + " " + copy );
				RandomAccessBitstreamArchive archive = (RandomAccessBitstreamArchive)ArchiveLoader.getInstance( basename, b );
				if ( copy ) archive = archive.copy();
				
				final int[] perm1 = new int[ archive.numberOfDocuments() ];

				for ( int i = 0; i < perm1.length; i++ ) perm1[ i ] = i;
				TestCollection.pseudoPermute( perm1, 4242 );

				System.err.println( "Testing ordered retrieval" );
				for ( int i = 0; i < perm1.length; i++ )
					assertEquals( Integer.toString( i ), TestCollection.testCollection2[ i ], archive.getDocumentByIndex( i ) );
				System.err.println( "Testing random retrieval" );
				for ( int i = 0; i < perm1.length; i++ )
					assertEquals( Integer.toString( i ), TestCollection.testCollection2[ perm1[ i ] ], archive.getDocumentByIndex( perm1[ i ] ) );
			}
		}
	}

	public void testByID() throws Exception {
		for( boolean b: new boolean[] { true, false } ) {
			final RandomAccessBitstreamArchive archive = (RandomAccessBitstreamArchive)ArchiveLoader.getInstance( basename, b );

			final int[] perm1 = new int[ archive.numberOfDocuments() ];

			for ( int i = 0; i < perm1.length; i++ ) perm1[ i ] = i;
			TestCollection.pseudoPermute( perm1, 0xbeefbeef );

			System.err.println( "Testing ordered retrieval" );
			for ( int i = 0; i < perm1.length; i++ )
				assertEquals( Integer.toString( i ), TestCollection.testCollection2[ i ], archive.getDocumentById( TestCollection.testCollection2[ i ].id() ) );
			System.err.println( "Testing random retrieval" );
			for ( int i = 0; i < perm1.length; i++ )
				assertEquals( Integer.toString( i ), TestCollection.testCollection2[ perm1[ i ] ], archive.getDocumentById( TestCollection.testCollection2[ perm1[ i ] ].id() ) );

		}
	}

	@SuppressWarnings("unchecked")
	public void testIterator() throws Exception {
		for( String s: new String[] { basename, basenameNoCounts, basenameNoDocLengths } ) 
			for( boolean b: new boolean[] { true, false } ) {

				try {
					Archive<ArrayDocumentSummary> archive = (Archive<ArrayDocumentSummary>)ArchiveLoader.getInstance( s, b );

					assertEquals( TestCollection.testCollection2.length, archive.numberOfDocuments() );
					assertEquals( TestCollection.numberOfWords2, archive.numberOfWords() );
					assertEquals( TestCollection.frequency2.length, archive.numberOfTerms() );

					for ( int i = 0; i < TestCollection.frequency2.length; i++ ) assertEquals( "Term " + i, TestCollection.frequency2[ i ], archive.frequency( i ) );

					int i = 0;
					for ( ArrayDocumentSummary doc : archive ) {
						assertEquals( TestCollection.testCollection2[ i ].size(), doc.size() );
						if ( doc.length() != -1 ) assertEquals( TestCollection.testCollection2[ i ].length(), doc.length() );
						doc.sort();
						for( int t = doc.size(); t-- != 0; ) {
							assertEquals( TestCollection.testCollection2[ i ].term( t ), doc.term( t ) );
							if ( doc.count( t ) != -1 ) assertEquals( TestCollection.testCollection2[ i ].count( t ), doc.count( t ) );
						}
						i++;
					}

				}
				catch ( Exception e ) {
					notTearDown = true;
					throw ( e );
				}
			}
	}

}
