package it.unimi.di.archive4j.tool;

/*		 
 * Copyright (C) 2008-2013 Alessio Orlandi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


import it.unimi.di.big.mg4j.index.snowball.PorterStemmer;
import it.unimi.di.big.mg4j.document.StringArrayDocumentCollection;
import it.unimi.di.archive4j.tool.MergePreprocessedData;
import it.unimi.di.archive4j.tool.Preprocess;
import it.unimi.dsi.fastutil.io.BinIO;
import it.unimi.dsi.io.FileLinesCollection;
import it.unimi.dsi.io.InputBitStream;
import it.unimi.dsi.util.Properties;
import it.unimi.dsi.util.StringMap;

import java.io.File;

import junit.framework.TestCase;
import it.unimi.di.archive4j.TestCollection;




public class PreprocessTest extends TestCase {
	
	protected String seqname, seqlastname;
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
		seqname = TestCollection.buildDocumentSequence( TestCollection.testParseDocSources );	
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
		
		final File seqfile = new File ( seqname );

		new File( seqname + "-out" + Preprocess.FREQUENCIES_EXTENSION ).delete();
		new File( seqname + "-out" + Preprocess.TERMS_EXTENSION ).delete();
		new File( seqname + "-out" + Preprocess.LENGTHS_EXTENSION ).delete();
		new File( seqname + "-out" + Preprocess.TERMS_EXTENSION ).delete();

		seqfile.delete();
	}
	
	public void testStemmer() throws Exception {
		String basename = File.createTempFile( "prep", null ).getAbsolutePath();
		StringArrayDocumentCollection coll = new StringArrayDocumentCollection( new String[] { "classes"} );
		
		Preprocess.run( basename, coll,  new PorterStemmer(), "text" );
		FileLinesCollection lines = new FileLinesCollection( Preprocess.batchName( basename, 0 ) + Preprocess.TERMS_EXTENSION, "UTF-8", false);
		
		assertTrue(lines.allLines().get( 0 ).equals( "class" ));	
	}
	
	@SuppressWarnings("unchecked")
	public void testSingle() throws Exception {
		Preprocess.main( ( "-S " + seqname + " -I " + "text " + seqname + "-out" ).split( " " ) );
		MergePreprocessedData.main( ( seqname + "-out " + seqname + "-out" ).split( " " ) );
		
		StringMap<CharSequence> termMap = (StringMap<CharSequence>)BinIO.loadObject ( seqname + "-out" + MergePreprocessedData.TERMMAP_EXTENSION );
		
		Properties properties = new Properties( seqname + "-out" + Preprocess.PROPERTIES_EXTENSION );
		int nDocs = properties.getInt( Preprocess.PropertyKeys.DOCUMENTS );
		final InputBitStream frequencies = new InputBitStream ( seqname +"-out" + Preprocess.FREQUENCIES_EXTENSION);
	
		final InputBitStream lengths = new InputBitStream ( seqname + "-out" + Preprocess.LENGTHS_EXTENSION );
		
		final int[] freqdata = new int[ termMap.size() ];
		final int[] lendata = new int [ nDocs ];
		
		for ( int i = 0; i < nDocs; i++) lendata[i] = lengths.readDelta();
		
		frequencies.readGammas( freqdata, termMap.size() );
		
		int[] realfreq = TestCollection.makeFrequency ( TestCollection.testParse );
		
		assertEquals ( TestCollection.testParse.length, nDocs );
		for ( int i = 0; i < nDocs; i++ )
			assertEquals ( TestCollection.testParse[i].length(), lendata[i] );

		for ( int i = 0; i < TestCollection.testTerms.length; i++ )
			assertEquals ( i, (int)termMap.getLong( TestCollection.testTerms[i] ) );
			

		
		assertEquals ( realfreq.length, freqdata.length );
		for(int i = 0; i < realfreq.length; i++ ) assertEquals ( realfreq[i], freqdata[i] );
	}
}
